@@ -6,17 +6,24 @@ from abc import ABC, abstractmethod
 from leuci_xyz import vectorthree as v3
 import math
 import numpy as np
-from . import invariant as ivm
+#from . import invariant as ivm
+from . import iv1
+from . import iv3
+from . import iv5
 
 ### Factory method for creation ##############################################################
-def create_interpolator(method, values, F, M, S, npy=None,degree=0, log_level=0):
+def create_interpolator(method, values, F, M, S, npy=None,degree=-1, log_level=0):
     if log_level > 0:
-        print("Interpolator:",method,F,M,S)
+        print("Interpolator:",method,F,M,S,"degree=",degree)
     intr = None
     if method == "linear":
         intr = Multivariate(values,F, M, S,npy, 1, log_level)
+    elif method == "cubic" and degree == 5: #not really cubic, but quintic
+        intr = Multivariate(values,F, M, S,npy, 5, log_level)
     elif method == "cubic":
         intr = Multivariate(values,F, M, S,npy, 3, log_level)
+    elif method == "bspline" and degree == 5:
+        intr = Bspline(values,F, M, S,npy, 5, log_level) # I am being cautious about degrees
     elif method == "bspline":
         intr = Bspline(values,F, M, S,npy, 3, log_level) 
     else: #nearest is default
@@ -26,7 +33,7 @@ def create_interpolator(method, values, F, M, S, npy=None,degree=0, log_level=0)
 
 ### Abstract class ############################################################################
 class Interpolator(ABC):
-    def __init__(self, values, F, M, S, npy=None,degree=0,log_level=0):
+    def __init__(self, values, F, M, S, npy=None,degree=-1,log_level=0):
         if npy != None:
             self._npy = npy
         else:
@@ -43,7 +50,7 @@ class Interpolator(ABC):
         self.degree = degree
         self._buffer = 28
         self.log_level = log_level         
-        self.h = 0.00001 #this is the iunterval for numerical differentiation
+        self.h = 0.0001 #this is the iunterval for numerical differentiation
                 
     @abstractmethod
     def get_value(self, x, y, z):
@@ -89,6 +96,22 @@ class Interpolator(ABC):
         #pos = self.get_pos_from_fms(u_f,u_m,u_s,F,M,S)
         #return self._values[pos]
     
+    def get_projection(self,slice):        
+        if slice == "xy":        
+            return self._npy.max(axis=(2))
+        elif slice == "yz":        
+            return self._npy.max(axis=(0))
+        elif slice == "zx":        
+            return self._npy.max(axis=(1))
+    
+    def get_cross_section(self,slice,layer):        
+        if slice == "xy":        
+            return self._npy[:,:,layer]
+        elif slice == "yz":        
+            return self._npy[layer,:,:]            
+        elif slice == "zx":        
+            return self._npy[:,layer,:]
+            
     def get_pos_from_fms(self, f, m, s,F=-1,M=-1,S=-1):
         use_f,use_m,use_s = self._F, self._M, self._S
         if F+M+S != -3:
@@ -228,7 +251,12 @@ class Multivariate(Interpolator):
     def init(self):
         self.points = self.degree + 1
         self.dimsize = math.pow(self.points, 3)
-        self.inv = ivm.InvariantVandermonde(self.degree)
+        if self.degree == 1:
+            self.inv = iv1.InvariantVandermonde()
+        elif self.degree == 3:
+            self.inv = iv3.InvariantVandermonde()
+        elif self.degree == 5:
+            self.inv = iv5.InvariantVandermonde()
         self.need_new = True
         self._xfloor = -1
         self._yfloor = -1
@@ -509,15 +537,15 @@ class Bspline(Interpolator):
             yWeight.append(0)
             zWeight.append(0)            
         #Compute the interpolation indices
-        i = int(math.floor(u_x) - self.degree / 2)
-        j = int(math.floor(u_y) - self.degree / 2)
-        k = int(math.floor(u_z) - self.degree / 2)
+        i = int(math.floor(u_x) - math.floor(self.degree / 2))
+        j = int(math.floor(u_y) - math.floor(self.degree / 2))
+        k = int(math.floor(u_z) - math.floor(self.degree / 2))
 
-        for l in range(self.degree+1):        
-            i,j,k=i+1,j+1,k+1        
+        for l in range(self.degree+1):                          
             xIndex[l] = i #if 71.1 passed in, for linear, we would want 71 and 72, 
             yIndex[l] = j
-            zIndex[l] = k            
+            zIndex[l] = k
+            i,j,k=i+1,j+1,k+1              
         #/* compute the interpolation weights */
         if (self.degree == 9):        
             xWeight = self.applyValue9(u_x, xIndex, weight_length)
